//
//  MainViewController.m
//  IvrJackExample
//
//  Created by rfid on 12-12-11.
//  Copyright 2012 . All rights reserved.
//

#import "MainViewController.h"
#import "IvrJackExampleAppDelegate.h"
#import <QuartzCore/QuartzCore.h>
#import <unistd.h>
#import "IvrUtils.h"

@implementation MainViewController

/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
        // Custom initialization
    }
    return self;
}
*/

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	NSArray *weekdayAry = [NSArray arrayWithObjects:@"Sun.", @"Mon.", @"Tues.", @"Wed.", @"Thur.", @"Fri.", @"Sat.", nil];
    
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:NSLocalizedString(@"YYYY-MM-dd  eee", nil)];
    [dateFormatter setShortWeekdaySymbols:weekdayAry];
    [dateFormatter setLocale:[[[NSLocale alloc] initWithLocaleIdentifier:@"en_US"] autorelease]];
    NSString *str = [dateFormatter stringFromDate:[NSDate date]];	
	lblDate.text = str;
    //
    if ([[UIScreen mainScreen] bounds].size.height == 568) {
        CGRect pos = imgBottom.frame;
        pos.origin.y = pos.origin.y + 88;
        imgBottom.frame = pos;
        pos = lblBottom.frame;
        pos.origin.y = pos.origin.y + 88;
        lblBottom.frame = pos;
    }
    //
    edtData.delegate = self;
    edtKey.delegate = self;
    edtBlockNo.delegate = self;
    edtData.layer.borderWidth = 1.0;
    edtData.layer.cornerRadius = 5.0;
}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}

-(IvrJackExampleAppDelegate *) getInstance {
	return (IvrJackExampleAppDelegate *)[[UIApplication sharedApplication]delegate];
}

-(void) updateView: (BOOL) visible {
	lblSN.hidden = visible;
	txtSN.hidden = visible;
	lblKey.hidden = visible;
	edtKey.hidden = visible;
	lblBlockNo.hidden = visible;
    edtBlockNo.hidden = visible;
	lblBlockData.hidden = visible;
    edtData.hidden = visible;
	btnQueryResult.hidden = visible;
    btnRead.hidden = visible;
    btnWrite.hidden = visible;
	lblUID.hidden = visible;
	txtUID.hidden = visible;
}

-(void) readerConnect:(NSString *) deviceSN {
	[self updateView: NO];
	txtUID.text = @"";
	txtSN.text = deviceSN;
	lblStatus.text = @"Welcome";
	[aivStatus stopAnimating];
}

-(void) readerDisconnect {
	lblStatus.text = @"You are not connected to the device!";
    [self updateView: YES];
	[aivStatus stopAnimating];
}

-(void) readerStatusChange: (IvrJackStatus) status {
	switch (status) {
		case ijsDetecting:
			[aivStatus startAnimating];
			lblStatus.text = @"Detecting...";
			break;
			
		case ijsRecognized:
			lblStatus.text = @"Recognized.";
			break;
			
		case ijsUnRecognized:
			lblStatus.text = @"Unrecognized!";
			break;
			
		case ijsPlugout:
			lblStatus.text = @"Plugout!";
			[self updateView: YES];
			break;

		default:
			break;
	}
}

-(void) updateUI: (NSString *) msg {
	if (msg != NULL)
		lblStatus.text = msg;
	else if (msg == NULL) {
		btnQueryResult.enabled = TRUE;
        btnRead.enabled = YES;
        btnWrite.enabled = YES;
		[aivStatus stopAnimating];
	}
}

-(void) updateLabelText:(NSString *)msg {
	dispatch_sync(dispatch_get_main_queue(), ^{
		[self updateUI: msg];
	});
}

-(void) showTagUID {
    NSString *sData = @"";
    for (int i = 0; i < ids[0]; i++) {
        sData = [sData stringByAppendingFormat: @"%02X", ids[i + 1]];
    }
	txtUID.text = sData;
}

-(void) readTagInfo {
	IvrJackService *this = [self getInstance].reader;
	int ret = [this readTag];
    cardType = 0xFF;
	if (ret == 0)
	{
		[self updateLabelText: @"Reading Tag..."];
		if (this.isCancel) {
			[self updateLabelText: NULL];
			return;
		}
		ret = [this getTagUID: ids cardType: &cardType];
		if (ret == 0 && cardType == 0xFF) {
            ret = 10;
		}		
	}
	if (ret == 0) {
		[self updateLabelText: @"Read success."];
		dispatch_sync(dispatch_get_main_queue(), ^{
			[self showTagUID];
		});
	}
	else if (ret == 10) {
		[self updateLabelText: @"Please device close to the tag, try again."];	
	}
	else if (ret > 10) {
		[self updateLabelText: @"Failed to read the tag block data!"];
	}
	else {
		[self updateLabelText: @"Device communication error, make sure that is plugged."];	
	}
	[self updateLabelText: NULL];
}

-(void) showTagData {
    NSString *sData = @"";
    int n = 4;
    if (cardType == 0x00) n = 16;
    for (int i = 0; i < n; i++) {
        sData = [sData stringByAppendingFormat: @"%02X", data[i]];
    }
    edtData.text = sData;
}

-(void) readTagBlock {
	IvrJackService *this = [self getInstance].reader;
	int ret = [this readTag];
    cardType = 0xFF;
	if (ret == 0)
	{
		[self updateLabelText: @"Reading Tag..."];
		if (this.isCancel) {
			[self updateLabelText: NULL];
			return;
		}
		ret = [this getTagUID: ids cardType: &cardType];
		if (ret == 0 && cardType == 0xFF) {
            ret = 10;
		}
        if (ret == 0 && cardType == 0x00)
        {
            //ret = [this tagAuthenticate: (blockNo / 4) pwds: pwds];
            //ret = [this tagAuthenticate: 1 sector: (blockNo / 4) pwds: pwds];
        }
        if (ret == 0)
        {
            if (cardType == 0x00)
                ret = [this tagReadBlockData: 1 secotr1:2 block1:0 sector2:2 block2:1 pwds:pwds data: data];
                //ret = [this tagReadData: (blockNo % 4) data: data];
            else
                ret = [this tagReadDataUL: blockNo length: 1 data: data];
        }
	}
	if (ret == 0) {
		[self updateLabelText: @"Read success."];
		dispatch_sync(dispatch_get_main_queue(), ^{
			[self showTagData];
		});
	}
	else if (ret == 10) {
		[self updateLabelText: @"Please device close to the tag, try again."];
	}
	else if (ret > 10) {
		[self updateLabelText: @"Failed to read the tag block data!"];
	}
	else {
		[self updateLabelText: @"Device communication error, make sure that is plugged."];
	}
	[self updateLabelText: NULL];
}

- (IBAction) btnQueryClick: (id) sender {
	[self getInstance].reader.isCancel = NO;
	btnQueryResult.enabled = FALSE;
	txtUID.text = @"";
	lblStatus.text = @"Start read tag";
	[aivStatus startAnimating];
	[self performSelectorInBackground: @selector(readTagInfo) withObject: nil];
}

- (IBAction) btnReadClick: (id) sender {
    @try {
        if ([edtKey.text length] != 12) {
            [IvrUtils ShowMessage:@"Key length error."];
            return;
        }
        blockNo = [edtBlockNo.text intValue];
        UInt8* bytes = [IvrUtils stringToByteArray: edtKey.text];
        for (int i = 0; i < 6; i++) {
            pwds[i] = bytes[i];
        }

    }
    @catch (NSException *exception) {
        [IvrUtils ShowMessage:@"Input field contain invalid value."];
        return;
    }
	[self getInstance].reader.isCancel = NO;
	btnRead.enabled = FALSE;
	lblStatus.text = @"Start read tag";
	[aivStatus startAnimating];
	[self performSelectorInBackground: @selector(readTagBlock) withObject: nil];
}

-(void) writeTagBlock {
	IvrJackService *this = [self getInstance].reader;
	int ret = [this readTag];
    cardType = 0xFF;
	if (ret == 0)
	{
		[self updateLabelText: @"Writting Tag..."];
		if (this.isCancel) {
			[self updateLabelText: NULL];
			return;
		}
		ret = [this getTagUID: ids cardType: &cardType];
		if (ret == 0 && cardType == 0xFF) {
            ret = 10;
		}
        if (ret == 0 && cardType == 0x00)
        {
            ret = [this tagAuthenticate: (blockNo / 4) pwds: pwds];
        }
        if (ret == 0)
        {
            if (cardType == 0x00)
                ret = [this tagWriteData: (blockNo % 4) data: data];
            else
                ret = [this tagWriteDataUL: blockNo length: 1 data: data];
        }
	}
	if (ret == 0) {
		[self updateLabelText: @"Write success."];
	}
	else if (ret == 10) {
		[self updateLabelText: @"Please device close to the tag, try again."];
	}
	else if (ret > 10) {
		[self updateLabelText: @"Failed to write the tag block data!"];
	}
	else {
		[self updateLabelText: @"Device communication error, make sure that is plugged."];
	}
	[self updateLabelText: NULL];
}

- (IBAction) btnWriteClick: (id) sender {
    @try {
        if ([edtKey.text length] != 12) {
            [IvrUtils ShowMessage:@"Key length error."];
            return;
        }
        if ([edtData.text length] < 8 || [edtData.text length] % 2 != 0) {
            [IvrUtils ShowMessage:@"Data field length error."];
            return;
        }
        blockNo = [edtBlockNo.text intValue];
        UInt8* bytes = [IvrUtils stringToByteArray: edtKey.text];
        for (int i = 0; i < 6; i++) {
            pwds[i] = bytes[i];
        }
        bytes = [IvrUtils stringToByteArray: edtData.text];
        for (int i = 0; i < [edtData.text length] / 2; i++) {
            data[i] = bytes[i];
        } 
    }
    @catch (NSException *exception) {
        [IvrUtils ShowMessage:@"Input field contain invalid value."];
        return;
    }
	[self getInstance].reader.isCancel = NO;
	btnWrite.enabled = FALSE;
	lblStatus.text = @"Start write tag";
	[aivStatus startAnimating];
	[self performSelectorInBackground: @selector(writeTagBlock) withObject: nil];
}

-(void) moveView: (CGRect) frame {
    int offset = frame.origin.y + 32 - (self.view.frame.size.height - 216.0); //键盘高度216

    NSTimeInterval animationDuration = 0.30f;
    [UIView beginAnimations:@"ResizeForKeyboard" context:nil];
    [UIView setAnimationDuration:animationDuration];
    
    //将视图的Y坐标向上移动offset个单位，以使下面腾出地方用于软键盘的显示
    if(offset > 0)
        self.view.frame = CGRectMake(0.0f, -offset, self.view.frame.size.width, self.view.frame.size.height);
    
    [UIView commitAnimations];
}

-(void)textFieldDidBeginEditing:(UITextField *)textField
{
    iPosY = self.view.frame.origin.y;
    [self moveView: textField.frame];
}

//当用户按下return键或者按回车键，keyboard消失
-(BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    return YES;
}

//输入框编辑完成以后，将视图恢复到原始状态
-(void)textFieldDidEndEditing:(UITextField *)textField
{
    self.view.frame = CGRectMake(0.0f, iPosY, self.view.frame.size.width, self.view.frame.size.height);
}

-(BOOL) textViewShouldBeginEditing:(UITextView *)textView {
    iPosY = self.view.frame.origin.y;
    [self moveView: textView.frame];
    return YES;
}

-(IBAction)textViewDidEndEditing:(UITextView *)textView {
    self.view.frame = CGRectMake(0.0f, iPosY, self.view.frame.size.width, self.view.frame.size.height);
}

-(BOOL) textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range replacementText:(NSString *)text {
    if([text isEqualToString:@"\n"]) {
        [textView resignFirstResponder];
        return NO;
    }
    return YES;
}

@end
